# -*- coding: UTF-8 -*-
# @Time     : 2/3/21 8:24 PM
# @Author   : Jackie
# @File     : utilGPG.py

import os
import gnupg
from .logger import logger


class UtilGPG:
    def __init__(self, verbose=False):
        """
        GPG工具类
        :param verbose: 是否打印详情日志
        """
        self.gpg = gnupg.GPG(verbose=verbose)
        self.pub_key, self.fingerprint, self.recipients = None, None, None

    def create_cert(self, passphrase, name_real, name_email, key_type="RSA", key_length=1024):
        self.gpg.encoding = 'utf-8'
        input_data = self.gpg.gen_key_input(passphrase=passphrase, name_real=name_real, name_email=name_email,
                                            key_type=key_type, key_length=key_length)
        key = self.gpg.gen_key(input_data)
        logger.info('已生成key：{0}'.format(key))
        return key

    def export_cert(self, key, secret=False, passphrase=None):
        if secret:
            assert passphrase is not None, 'the param `passphrase` is required'
            logger.info(passphrase)
            ascii_armored_private_keys = self.gpg.export_keys(key, secret=True, passphrase=passphrase)
            with open('%s_private.asc' % key, mode='w') as f1:
                f1.write(ascii_armored_private_keys)
        else:
            ascii_armored_public_keys = self.gpg.export_keys(key)
            with open('%s_public.asc' % key, mode='w') as f1:
                f1.write(ascii_armored_public_keys)

    def get_cert(self, path):
        logger.info('Start get pub cert[%s].' % path)
        os.system('gpg --import %s' % path)
        with open(path, 'r') as f:
            pub_key = ascii(f.read())

        self.pub_key = self.gpg.import_keys(pub_key)
        self.fingerprint = self.gpg.list_keys()
        logger.info('Public cert fingerprint[%s], pub_key[%s].' % (str(self.fingerprint), self.pub_key.results))
        self.recipients = self.fingerprint[0]['uids'][0]

    def encrypt_file(self, file_path):
        logger.info('Start encrypt file[%s].' % file_path)
        stream = open(file_path, mode='rb')
        filepath, file_name = os.path.split(file_path)
        output_file_name = "{0}.gpg".format(file_name)
        output_file_path = os.path.join(filepath, output_file_name)
        self.gpg.encrypt_file(stream, always_trust=True, recipients=self.recipients, output=output_file_path, armor=False)
        logger.info('Encrypt file success[%s], result[%s].' % (output_file_path, output_file_path))
        return output_file_path

    def decrypted_file(self, file_path, out_file, private_cert_path, passphrase=None, ):
        # 导入私钥
        command = "gpg --import {0}".format(private_cert_path)
        print(command)
        os.system(command)
        filepath, file_name = os.path.split(file_path)
        stream = open(file_name, mode='rb')
        decrypted_data = self.gpg.decrypt_file(stream, passphrase=passphrase)
        print('解析成功')
        # 返回值为 gnupg.Crypt 类型。
        with open(out_file, 'w') as f:
            f.write(str(decrypted_data))



# -*- coding: utf-8 -*-
"""
@Time    : 2019-12-26 19:40
@Author  : jackie
"""

from xlrd import open_workbook


class MyXls:
    def __init__(self, file_path):
        self.file = open_workbook(file_path)
        self.sheet_names = self.file.sheet_names()

    def get_sheet(self, sheet_name):
        sheet = self.file.sheet_by_name(sheet_name)
        row_num = sheet.nrows

        sheet_data = {}

        for i in range(1, row_num):
            row_data = sheet.row_values(rowx=i)
            foreign_key = row_data[0]
            if foreign_key not in sheet_data.keys():
                sheet_data[foreign_key] = []
            data = {}
            for index, key in enumerate(sheet.row_values(0)):
                data[key] = row_data[index]
                data['host'] = sheet_name
            sheet_data[foreign_key].append(data)
        return sheet_data



# -*- coding: UTF-8 -*-
# @Time     : 2/3/21 8:24 PM
# @Author   : Jackie
# @File     : utilSftp.py

import paramiko
import os


class SshConnectError(Exception):
    pass


class UtilSftp:
    def __init__(self, config):  # 本地密钥文件路径
        self.host = config['host']
        if config['key_file']:
            self.pkey = paramiko.RSAKey.from_private_key_file(config['key_file'])
        else:
            self.pkey = config['key_file']
        self.ssh = self.__ssh_conn(self.host, config['username'], config['password'], self.pkey, int(config['port']))
        self.sftp = self.__sftp_conn()

    def close(self):
        if hasattr(self.ssh, "close"):
            self.ssh.close()

    def __ssh_conn(self, host, username, password, pkey, port):
        ssh = paramiko.SSHClient()  # 创建一个SSH客户端client对象
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        try:
            if self.pkey:
                ssh.connect(hostname=host, port=int(port), username=username, pkey=pkey)  # 免密登陆方式
            else:
                ssh.connect(hostname=host, port=int(port), username=username, password=password)  # 密码认证
        except:
            raise SshConnectError("SSH Connect %s Error!" % host)
        else:
            return ssh

    # 返回sftp通道实例对象 方法
    def __sftp_conn(self):
        transport = self.ssh.get_transport()  # 1.先ssh连上，2.再建立通道
        sftp = paramiko.SFTPClient.from_transport(transport)  # 创建一个已连通的SFTP客户端通道。
        return sftp

    # 执行命令方法
    def exe_command(self, cmd, timeout=300):
        _, stdout, stderr = self.ssh.exec_command(cmd, timeout=timeout)
        try:
            channel = stdout.channel
            # print('channel',channel)
            exit_code = channel.recv_exit_status()
            # print('exit_code',exit_code)报错返回码是127，没有报错是0
            stdout = stdout.read().strip()
            stderr = stderr.read().strip()
            return {"status": 1, "stdout": stdout, "stderr": stderr, 'exit_code': exit_code}
        except:
            return {"status": 0, "stdout": stdout, "stderr": stderr, 'exit_code': 127}

    # 文件上传下载方法
    def trans_file(self, local_path, remote_path, action, **kwargs):
        try:
            if action == 'push':
                dirname = os.path.dirname(remote_path)
                self.exe_command("mkdir -p %s" % dirname)
                self.sftp.put(local_path, remote_path)
                return {"status": 1, "message": 'sftp %s %s success!' % (self.host, action)}
            elif action == "pull":
                dirname = os.path.dirname(local_path)
                if not os.path.exists(dirname):
                    os.makedirs(dirname)
                # if os.path.exists(local_path):
                #     os.remove(local_path)
                self.sftp.get(remote_path, local_path)
                return {"status": 1, "stdout": 'sftp %s %s success!' % (self.host, action), "stderr": ""}
        except Exception as e:
            return {"status": 0, "stderr": 'sftp %s %s failed %s' % (self.host, action, str(e)), "stdout": ""}

    @staticmethod
    def iter_local_path(abs_path):
        """遍历本机该目录中所以的文件，并返回"""
        result = set([])
        for j in os.walk(abs_path):
            print(j)
            base_path = j[0]
            file_list = j[2]
            for k in file_list:
                p = os.path.join(base_path, k)
                result.add(p)
        return result

    def iter_remote_path(self, abs_path):
        """获取远程主机abs_path下的所以文件"""
        result = set([])
        try:
            stat = str(self.sftp.lstat(abs_path))
            print('stat', stat)
        except FileNotFoundError:
            return result
        else:
            if stat.startswith("d"):
                file_list = self.exe_command("ls %s" % abs_path)["stdout"].decode(encoding='utf-8').strip().splitlines()
                # Python strip() 方法用于移除字符串头尾指定的字符（默认为空格或换行符）或字符序列。注意：该方法只能删除开头或是结尾的字符，不能删除中间部分的字符。 Python splitlines(
                # ) 按照行('\r', '\r\n', \n')分隔，返回一个包含各行作为元素的列表，如果参数 keepends 为(默认值) False，不包含换行符，如果为 True，则保留换行符。

                for j in file_list:
                    p = os.path.join(abs_path, j)
                    result.update(self.iter_remote_path(p))  # 合并 并集U
            else:
                result.add(abs_path)
        return result



# -*- coding: UTF-8 -*-
# @Time     : 2020/7/10 10:22
# @Author   : Jackie
# @File     : handlerRedis.py
import traceback

import redis
from .read_config import apollo_reader
from .logger import logger


class HandlerRedis:
    def __init__(self, tag=None):
        if not tag:
            self.config = {
                'host': apollo_reader.get_value('redis.host'),
                'port': int(apollo_reader.get_value('redis.port')),
                'password': apollo_reader.get_value('redis.password'),
                'db': apollo_reader.get_value('redis.db'),
                'decode_responses': True,
            }
        else:
            self.config = {
                'host': apollo_reader.get_value(f'redis.{tag}.host'),
                'port': int(apollo_reader.get_value(f'redis.{tag}.port')),
                'password': apollo_reader.get_value(f'redis.{tag}.password'),
                'db': apollo_reader.get_value(f'redis.{tag}.db'),
                'decode_responses': True,
            }

        self.default_ex = 3600 * 24
        self.pool = redis.ConnectionPool(**self.config)
        self.rds = redis.Redis(connection_pool=self.pool)
        self.pipe = self.rds.pipeline()
        logger.info('[%s] connected.' % self)

    def __str__(self):
        return 'Redis[{host}:{port}], db[{db}]'.format(**self.config)

    def set_ex(self, key, value, ex=3600*24):
        if ex == self.default_ex:
            ex = self.rds.ttl(key)
        if ex > 0:
            self.rds.setex(key, ex, value)
        else:
            self.rds.setex(key, self.default_ex, value)

    def lpush(self, key, data):
        self.rds.lpush(key, data)

    def llen(self, key):
        self.rds.llen(key)


class RedisHandler:
    """
    提供给主营的缓存操作工具
    """
    def __init__(self, host, port, password, db, *args, **kwargs):
        self.config = {
            'host': host,
            'port': port,
            'password': password,
            'db': db,
            'decode_responses': True,
        }
        self.args = args
        self.kwargs = kwargs
        self.pool = redis.ConnectionPool(**self.config)
        self.rds = redis.Redis(connection_pool=self.pool)
        self.pipe = self.rds.pipeline()
        self.support_type = ['string', 'hash', 'list', 'set', 'zset']

    def __str__(self):
        return 'Redis[{host}:{port}], db[{db}]'.format(**self.config)

    def exec(self):
        commands = self.kwargs.get('commands')
        result = []
        for command in commands:
            key_type = command.get('type')
            if key_type not in self.support_type:
                result.append(RuntimeError(f'Not support type, choice in [{self.support_type}]').__repr__())
                continue
            func_name = command.get('operate')
            key_name = command.get('name')
            args = command.get('args')
            kwargs = command.get('kwargs')

            rds_command = 'self.rds.%s("%s",' % (func_name, key_name)
            if args:
                rds_command += '*args,'
            if kwargs:
                rds_command += '**kwargs,'
            rds_command += ')'
            try:
                logger.info(f'RedisHandler exec command:{rds_command}.')
                result.append(eval(rds_command))
            except Exception as e:
                logger.error(f'exec{rds_command} redis throw exception[{e}]')
                logger.error(f'traceback.format_exc():\n{traceback.format_exc()}')
                result.append(e.__repr__())
        return result


if __name__ == '__main__':
    config = {
        'host': 'r-d9jabfe0f2736274.redis.ap-southeast-5.rds.aliyuncs.com',
        'port': 6379,
        'password': '1qaz@WSX',
        'db': 3,
        'commands': [
            {'type': 'string', 'operate': 'set', 'name': 'jackie', 'kwargs': {'value': 'abc'}},
            {'type': 'string', 'operate': 'get', 'name': 'jackie', },
            {'type': 'hash', 'operate': 'hset', 'name': 'jackie_hash', 'kwargs': {'key': 'kkk', 'value': 'vvv'}},
            {'type': 'list', 'operate': 'lpush', 'name': 'jackie_list', 'args': [1, 2, 3]},
            {'type': 'set', 'operate': 'sadd', 'name': 'jackie_set', 'args': [1, 2, 3]},
            {'type': 'zset', 'operate': 'zadd', 'name': 'jackie_zset', 'kwargs': {'mapping': {'a': 1, 'b': 2, 'c': 3}}},
            {'type': 'string', 'operate': 'get', 'name_test': 'jackie'},
            {'type': 'string', 'operate': 'get_test', 'name': 'jackie'},
            {'type': 'string_test', 'operate': 'get_test', 'name': 'jackie'},
        ],
    }
    rds = RedisHandler(**config)
    print(rds.exec())


def get_rds_handler(tag=None):
    return HandlerRedis(tag)



# -*- encoding=utf-8 -*-
import logging
import os
import time


# 定义日志输出
def init_log():
    log = logging.getLogger()
    log.setLevel(logging.INFO)

    if not os.path.exists('./logs'):
        os.makedirs('./logs')

    fh = logging.FileHandler(time.strftime('./logs/%Y%m%d', time.localtime(time.time())) + '.log')
    fh.setFormatter(logging.Formatter("%(asctime)s - [%(levelname)s] %(filename)s[line:%(lineno)d]: %(message)s"))

    sh = logging.StreamHandler()
    sh.setFormatter(logging.Formatter("%(asctime)s - [%(levelname)s] %(filename)s[line:%(lineno)d]: %(message)s"))

    log.addHandler(fh)
    log.addHandler(sh)

    return log


logger = init_log()



# -*- coding: UTF-8 -*-
# @Time     : 2/7/21 6:25 PM
# @Author   : Jackie
# @File     : utilFileTrans.py

from .logger import logger
from .utilFtp import UtilFtp
from .utilSftp import UtilSftp


class UtilFileTrans:
    def __init__(self, trans_type, config):
        """

        :param config: dict.
            if trans_type == ftp: need params[host, port, username, password, debug_level*]
            elif trans_type == sftp: need params[host, port, username, password, key_path*]
        """
        if trans_type not in ('ftp', 'sftp'):
            logger.error('Un support transfer type[%s].' % trans_type)
            raise RuntimeError
        if trans_type == 'ftp':
            self.util_file_transfer = UtilFtp(config)
        elif trans_type == 'sftp':
            self.util_file_transfer = UtilSftp(config)

    def trans_file(self, source_path, tar_path, action, **kwargs):
        self.util_file_transfer.trans_file(source_path, tar_path, action, **kwargs)





# -*- coding: UTF-8 -*-
# @Time     : 2020/8/27 11:00
# @Author   : Jackie
# @File     : handlerMysql.py

import pymysql
from dbutils.pooled_db import PooledDB
from .read_config import apollo_reader
from .logger import logger


class MyDB:
    def __init__(self, database='default'):
        apollo_configurations = apollo_reader.get_configurations()
        mysql_hosts = [key for key in apollo_configurations.keys() if key.startswith('mysql') and key.endswith('host')]
        databases = [database.split('.')[1] for database in mysql_hosts]

        self.database = database
        if self.database not in databases:
            self.database = 'default'

        db_config = {
            'host': apollo_reader.get_value('mysql.%s.host' % str(self.database)),
            'user': apollo_reader.get_value('mysql.%s.user' % str(self.database)),
            'passwd': apollo_reader.get_value('mysql.%s.pwd' % str(self.database)),
            'port': int(apollo_reader.get_value('mysql.%s.port' % str(self.database))),
            'db': apollo_reader.get_value('mysql.%s.database' % str(self.database)),
            'autocommit': True,
        }
        pool = PooledDB(pymysql, 5, **db_config)
        self.db = pool.connection()
        self.cursor = self.db.cursor()
        self.cursor_dict = self.db.cursor(cursor=pymysql.cursors.DictCursor)

    def execute_sql(self, sql, is_dict=False, is_commit=True):
        self.ping()
        if is_dict:
            self.cursor_dict.execute(sql)
            cursor = self.cursor_dict
        else:
            self.cursor.execute(sql)
            cursor = self.cursor
        if is_commit:
            self.db.commit()
        return cursor

    def get_all(self, is_dict=False):
        if is_dict:
            value = self.cursor_dict.fetchall()
        else:
            value = self.cursor.fetchall()
        return value

    def get_one(self):
        value = self.cursor.fetchone()
        if value is not None:
            try:
                return value[0]
            except IndexError:
                return None
        return value

    def commit(self):
        self.db.commit()

    def ping(self, reconnect=True):
        self.db.ping(reconnect)

    def close_db(self):
        self.db.close()
        logger.info("Database closed!")



# -*- coding:utf-8 -*-

import json
import logging
import threading
import time
import requests


class ApolloClient(object):
    def __init__(self, app_id, cluster='default', config_server_url='http://localhost:8080', interval=5, ip=None):
        self.config_server_url = config_server_url
        self.appId = app_id
        self.cluster = cluster
        self.timeout = 60
        self.interval = interval
        self.init_ip(ip)
        self._stopping = False
        self._cache = {}
        self._notification_map = {'application': -1}

    def init_ip(self, ip):
        if ip:
            self.ip = ip
        else:
            import socket
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                s.connect(('8.8.8.8', 53))
                ip = s.getsockname()[0]
            finally:
                s.close()
            self.ip = ip

    def get_value(self, key, default_val=None, namespace='application', auto_fetch_on_cache_miss=False):
        if namespace not in self._notification_map:
            self._notification_map[namespace] = -1
            logging.getLogger(__name__).info("Add namespace '%s' to local notification map", namespace)

        if namespace not in self._cache:
            self._cache[namespace] = {}
            logging.getLogger(__name__).info("Add namespace '%s' to local cache", namespace)
            self._long_poll()
        if key in self._cache[namespace]:
            return self._cache[namespace][key]
        else:
            if auto_fetch_on_cache_miss:
                return self._cached_http_get(key, default_val, namespace)
            else:
                return default_val

    def start(self):
        if len(self._cache) == 0:
            self._long_poll()
        t = threading.Thread(target=self._listener)
        t.start()

    def stop(self):
        self._stopping = True
        logging.getLogger(__name__).info("Stopping listener...")

    def _cached_http_get(self, key, default_val, namespace='application'):
        url = '{}/configfiles/json/{}/{}/{}?ip={}'.format(self.config_server_url, self.appId, self.cluster, namespace,
                                                          self.ip)
        r = requests.get(url)
        if r.ok:
            data = r.json()
            self._cache[namespace] = data
            logging.getLogger(__name__).info('Updated local cache for namespace %s', namespace)
        else:
            data = self._cache[namespace]

        if key in data:
            return data[key]
        else:
            return default_val

    def _uncached_http_get(self, namespace='application'):
        url = '{}/configs/{}/{}/{}?ip={}'.format(self.config_server_url, self.appId, self.cluster, namespace, self.ip)
        r = requests.get(url)
        if r.status_code == 200:
            data = r.json()
            self._cache[namespace] = data['configurations']
            logging.getLogger(__name__).info('Updated local cache for namespace %s release key %s: %s',
                                             namespace, data['releaseKey'],
                                             repr(self._cache[namespace]))

    def _long_poll(self):
        url = '{}/notifications/v2'.format(self.config_server_url)
        notifications = []
        for key in self._notification_map:
            notification_id = self._notification_map[key]
            notifications.append({
                'namespaceName': key,
                'notificationId': notification_id
            })

        r = requests.get(url=url, params={
            'appId': self.appId,
            'cluster': self.cluster,
            'notifications': json.dumps(notifications, ensure_ascii=False)
        }, timeout=self.timeout)

        logging.getLogger(__name__).debug('Long polling returns %d: url=%s', r.status_code, r.request.url)

        if r.status_code == 304:
            # no change, loop
            logging.getLogger(__name__).debug('No change, loop...')
            return

        if r.status_code == 200:
            data = r.json()
            for entry in data:
                ns = entry['namespaceName']
                nid = entry['notificationId']
                logging.getLogger(__name__).info("%s has changes: notificationId=%d", ns, nid)
                self._uncached_http_get(ns)
                self._notification_map[ns] = nid
        else:
            logging.getLogger(__name__).warn('Sleep...')
            time.sleep(self.timeout)

    def _listener(self):
        logging.getLogger(__name__).info('Entering listener loop...')
        while not self._stopping:
            try:
                self._long_poll()
            except Exception:
                pass
            finally:
                time.sleep(self.interval)
        logging.getLogger(__name__).info("Listener stopped!")

    def get_configurations(self, namespace='application'):
        return self._cache[namespace]



# -*- coding: UTF-8 -*-
# @Time     : 2020/7/10 10:22
# @Author   : Jackie
# @File     : handlerMongo.py
from .logger import logger
import pymongo


class MongoHandler:
    def __init__(self, conn_config):
        self.client = pymongo.MongoClient(conn_config)
        self.dbs = {}
        self.coll_list = {}
        logger.info(f'MongoClient conn success. {self.client}')

    def get_db(self, db_name):
        if db_name in self.dbs:
            return self.dbs[db_name]['obj']
        db = self.client.get_database(db_name)
        self.dbs[db_name] = {}
        self.dbs[db_name]['obj'] = db
        return db

    def get_coll(self, db_name, coll_name):
        db = self.get_db(db_name)
        db_data = self.dbs[db_name]
        if coll_name in db_data:
            return db_data[coll_name]
        coll = db[coll_name]
        db_data[coll_name] = coll
        return coll

    def get_result(self, db_name, coll_name, params, *args, **kwargs):
        logger.info(f'Query mongo: db_name:{db_name}, coll_name:{coll_name}, params:{args, kwargs}')
        coll = self.get_coll(db_name, coll_name)
        return list(coll.find(params, *args, **kwargs))


if __name__ == '__main__':
    m = MongoHandler('mongodb://root:5T8JbklD9v5ylAZ@s-d9j40f279e3cd0c4.mongodb.ap-southeast-5.rds.aliyuncs.com:3717,s-d9jc65211d9803b4.mongodb.ap-southeast-5.rds.aliyuncs.com:3717/sdk-backend?authSource=admin')
    print(m.client.get_database('userData'))
    r = m.get_result('userData', 'user', {'user_id': 100})
    print(r)
